<?php
// $Id: textsize.block.inc,v 1.1.2.6 2009/10/02 12:43:07 christianzwahlen Exp $

/**
 * @file
 * block, page and theme functions.
 */

/**
 * Returns an array of allowed values.
 *
 * @return
 *   An array of allowed values.
 */
function textsize_allowed_values() {
  $textsize_increment = variable_get('textsize_increment', 5);
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_minimum = variable_get('textsize_minimum', 50);
  $textsize_maximum = variable_get('textsize_maximum', 150);
  $textsize_start = ($textsize_normal - (round((($textsize_normal - $textsize_minimum) / $textsize_increment), 0) * $textsize_increment));
  $textsize_end = ($textsize_normal + (round((($textsize_maximum - $textsize_normal) / $textsize_increment), 0) * $textsize_increment));
  $textsize_allowed_values = array();
  for ($value = $textsize_start; $value <= $textsize_end; $value = $value + $textsize_increment ) {
    $textsize_allowed_values[] = $value;
  };
  return $textsize_allowed_values;
}

/**
 * Menu callback; increase the textsize, then redirects to the previous page.
 */
function textsize_increase() {
  $textsize_link_type = variable_get('textsize_link_type', 'variable');
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_increment = variable_get('textsize_increment', 5);
  $textsize_maximum = variable_get('textsize_maximum', 150);
  $textsize_maximum_display = textsize_display($textsize_maximum, 0);
  $textsize_post = array();
  $textsize_cookie = array();
  // allowed values
  $textsize_allowed = textsize_allowed_values();
  // check the type and the content
  if (in_array($_COOKIE['textsize'], $textsize_allowed)) {
    $textsize_cookie['textsize'] = (int) filter_xss($_COOKIE['textsize']);
  }
  // calculation of text size
  $ts_value = ($textsize_cookie['textsize'] + $textsize_increment);
  $ts_value_display = textsize_display(($textsize_cookie['textsize'] + $textsize_increment), 0);
  // set session/cookie
  if (!isset($_COOKIE['textsize'])) {
    drupal_set_message(t('The text size have not been saved, because your browser do not accept cookies.'), 'error');
  }
  elseif ($textsize_link_type == 'fix' && $textsize_cookie['textsize'] == ($textsize_normal + $textsize_increment)) {
    drupal_set_message(t('The text size maximum is: %ts_max%.', array('%ts_max' => textsize_display(($textsize_normal + $textsize_increment), 0))), 'error');
  }
  elseif ($textsize_cookie['textsize'] == $textsize_maximum) {
    drupal_set_message(t('The text size maximum is: %ts_max%.', array('%ts_max' => $textsize_maximum_display)), 'error');
  }
  elseif ($textsize_link_type == 'fix' && $textsize_cookie['textsize'] == $textsize_normal || $textsize_link_type == 'fix' && $textsize_cookie['textsize'] == ($textsize_normal - $textsize_increment)) {
    $_SESSION['textsize'] = ($textsize_normal + $textsize_increment);
    setcookie("textsize", ($textsize_normal + $textsize_increment), time()+31536000, "/", "");
    drupal_set_message(t('The text size have been saved as %ts_value%.', array('%ts_value' => textsize_display(($textsize_normal + $textsize_increment), 0),)));
  }
  else {
    $_SESSION['textsize'] = $ts_value;
    setcookie("textsize", $ts_value, time()+31536000, "/", "");
    drupal_set_message(t('The text size have been saved as %ts_value%.', array('%ts_value' => $ts_value_display)));
  }
  textsize_clear_cache();
  drupal_goto();
}

/**
 * Menu callback; decrease the textsize, then redirects to the previous page.
 */
function textsize_decrease() {
  $textsize_link_type = variable_get('textsize_link_type', 'variable');
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_increment = variable_get('textsize_increment', 5);
  $textsize_minimum = variable_get('textsize_minimum', 50);
  $textsize_minimum_display = textsize_display($textsize_minimum, 0);
  $textsize_post = array();
  $textsize_cookie = array();
  // allowed values
  $textsize_allowed = textsize_allowed_values();
  // check the type and the content
  if (in_array($_COOKIE['textsize'], $textsize_allowed)) {
    $textsize_cookie['textsize'] = (int) filter_xss($_COOKIE['textsize']);
  }
  // calculation of text size
  $ts_value = (($textsize_cookie['textsize'] - $textsize_increment));
  $ts_value_display = textsize_display(($textsize_cookie['textsize'] - $textsize_increment), 0);
  // set session/cookie
  if (!isset($_COOKIE['textsize'])) {
    drupal_set_message(t('The text size have not been saved, because your browser do not accept cookies.'), 'error');
  }
  elseif ($textsize_link_type == 'fix' && $textsize_cookie['textsize'] == ($textsize_normal - $textsize_increment)) {
    drupal_set_message(t('The text size minimum is: %ts_min%.', array('%ts_min' =>  textsize_display(($textsize_normal - $textsize_increment), 0))), 'error');
  }
  elseif ($textsize_cookie['textsize'] == $textsize_minimum) {
    drupal_set_message(t('The text size minimum is: %ts_min%.', array('%ts_min' => $textsize_minimum)), 'error');
  }
  elseif ($textsize_link_type == 'fix' && $textsize_cookie['textsize'] == $textsize_normal || $textsize_link_type == 'fix' && $textsize_cookie['textsize'] == ($textsize_normal + $textsize_increment)) {
    $_SESSION['textsize'] = ($textsize_normal - $textsize_increment);
    setcookie("textsize", ($textsize_normal - $textsize_increment), time()+31536000, "/", "");
    drupal_set_message(t('The text size have been saved as %ts_value%.', array('%ts_value' => textsize_display(($textsize_normal - $textsize_increment), 0),)));
  }
  else {
    $_SESSION['textsize'] = $ts_value;
    setcookie("textsize", $ts_value, time()+31536000, "/", "");
    drupal_set_message(t('The text size have been saved as %ts_value%.', array('%ts_value' => $ts_value_display)));
  }
  textsize_clear_cache();
  drupal_goto();
}

/**
 * Menu callback; change the textsize to normal, then redirects to the previous page.
 */
function textsize_normal() {
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_normal_display = textsize_display($textsize_normal, 0);
  // set session/cookie
  if (!isset($_COOKIE['textsize'])) {
    drupal_set_message(t('The text size have not been saved, because your browser do not accept cookies.'), 'error');
  }
  else {
    $_SESSION['textsize'] = $textsize_normal;
    setcookie("textsize", $textsize_normal, time()+31536000, "/", "");
    drupal_set_message(t('The text size have been saved as %textsize_normal%.', array('%textsize_normal' => $textsize_normal_display)));
  }
  textsize_clear_cache();
  drupal_goto();
}

/**
 * Menu callback; change the textsize to the cookie value, then redirects to the previous page.
 */
function textsize_set() {
  $textsize_normal = variable_get('textsize_normal', 100);
  $textsize_post = array();
  $textsize_cookie = array();
  // allowed values
  $textsize_allowed = textsize_allowed_values();
  // check the type and the content
  if (in_array($_COOKIE['textsize'], $textsize_allowed)) {
    $textsize_cookie['textsize'] = (int) filter_xss($_COOKIE['textsize']);
  }
  // set session/cookie
  if (!isset($_COOKIE['textsize'])) {
    drupal_set_message(t('The text size have not been saved, because your browser do not accept cookies.'), 'error');
  }
  elseif (isset($_COOKIE['textsize'])) {
    $_SESSION['textsize'] = $textsize_cookie['textsize'];
    setcookie("textsize", $textsize_cookie['textsize'], time()+31536000, "/", "");
    drupal_set_message(t('The text size have been saved as %textsize%.', array('%textsize' => textsize_display($textsize_cookie['textsize'], 0))));
  }
  textsize_clear_cache();
  drupal_goto();
}

/**
 * Clear defined cache tables
 */
function textsize_clear_cache() {
  $tables = array(
    'cache_page',
    'cache_block',
  );
  foreach ($tables as $table) {
    cache_clear_all('*', $table, TRUE);
  }
}

/**
 * Returns an array of available values for the select element in the block text size.
 *
 * @return
 *   An array of available values, for selecting the text size.
 *
 * @see textsize_form()
 */
function textsize_block_form_options() {
  $textsize_link_type = variable_get('textsize_link_type', 'variable');
  $textsize_increment = variable_get('textsize_increment', 5);
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_minimum = variable_get('textsize_minimum', 50);
  $textsize_maximum = variable_get('textsize_maximum', 150);
  $textsize_increment_display = textsize_display($textsize_increment, 0);
  $textsize_normal_display = textsize_display($textsize_normal, 0);
  $textsize_minimum_display = textsize_display($textsize_minimum, 0);
  $textsize_maximum_display = textsize_display($textsize_maximum, 0);
  if ($textsize_link_type == 'fix') {
    $textsize_start = ($textsize_normal - $textsize_increment);
    $textsize_end = ($textsize_normal + $textsize_increment);
  }
  elseif ($textsize_link_type == 'variable') {
    $textsize_start = ($textsize_normal - (round((($textsize_normal - $textsize_minimum) / $textsize_increment), 0) * $textsize_increment));
    $textsize_end = ($textsize_normal + (round((($textsize_maximum - $textsize_normal) / $textsize_increment), 0) * $textsize_increment));

  }
  $options = array();
  for ($value = $textsize_start; $value <= $textsize_end; $value = $value + $textsize_increment) {
    $options[$value] = textsize_display($value, 0);
  };
  return $options;
}

/**
 * Implementation of hook_form().
 *
 * Generate the select form for the block.
 *
 * @see textsize_block()
 */
function textsize_form($form) {
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_increment = variable_get('textsize_increment', 5);
  $textsize_minimum = variable_get('textsize_minimum', 50);
  $textsize_maximum = variable_get('textsize_maximum', 150);
  $display_current_textsize = variable_get('textsize_display_current_textsize', 'display');
  $textsize_current = textsize_get_current($value = 'int');
  $options = textsize_block_form_options();
  $textsize_post = array();
  $textsize_cookie = array();
  // allowed values
  $textsize_allowed = textsize_allowed_values();
  // check the type and the content
  if (isset($_COOKIE['textsize']) && isset($_POST['textsize_select'])) {
    if (in_array($_POST['textsize_select'], $textsize_allowed)) {
      $textsize_post['textsize'] = (string) filter_xss($_POST['textsize_select']);
    }
    // set session/cookie
    if ($textsize_post['textsize'] <= $textsize_maximum && $textsize_post['textsize'] >= $textsize_minimum) {
      setcookie("textsize", $textsize_post['textsize'], time()+31536000, "/", "");
      $_SESSION['textsize'] = $textsize_post['textsize'];
      drupal_set_message(t('The text size have been saved as %textsize_post%.', array('%textsize_post' =>  textsize_display($textsize_post['textsize'], 0))));
      textsize_clear_cache();
    }
  }
  elseif (!isset($_COOKIE['textsize']) && isset($_POST['textsize_select'])) {
    drupal_set_message(t('The text size have not been saved, because your browser do not accept cookies.'), 'error');
  }
  $form = array();
  $form['textsize_select'] = array(
    '#type' => 'select',
  //'#title' => t('Text Size'),
    '#name' => 'textsize_select',
    '#options' => $options,
    '#default_value' => variable_get('textsize_select', $textsize_current),
    '#weight' => 0,
  //'#description' => t('Select the text size.'),
  );
  $form['textsize_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 1,
  );
  $form['#skip_duplicate_check'] = TRUE;
  return $form;
}

/**
 * Implementation of theme().
 *
 * @see textsize_form()
 */
function theme_textsize_form($form) {
  $output = drupal_render($form);
  return $output;
}

/**
 * Implementation of hook_theme().
 */
function textsize_theme() {
  $arguments = array(
    'dest' => NULL,
    'list_inline' => NULL,
    'link_type' => NULL,
    'display_links' => NULL,
    'increment' => NULL,
    'normal' => NULL,
    'display_current_textsize' => NULL,
    'textsize' => NULL,
  );
  return array(
    'textsize_text' => array(
      'template' => 'textsize-text',
      'arguments' => $arguments,
    ),
    'textsize_image' => array(
      'template' => 'textsize-image',
      'arguments' => $arguments,
    ),
    'textsize_form' => array(
      'template' => 'textsize-form',
      'arguments' => array(
        'form' => NULL,
        'textsize_form' => NULL,
        'display_current_textsize' => NULL,
        'textsize' => NULL,
      )
    ),
  );
}

/**
 * Process variables for textsize-text.tpl.php.
 *
 * The $variables array contains the following arguments:
 * - $block_title: Block title: "Text Size" or "Zoom".
 * - $dest: The drupal function drupal_get_destination().
 * - $list_inline: Display the ul list "inline" or not.
 * - $link_type: Return "variable" for variable or fix.
 * - $display_links: Show or hide ("display_hidden") the text in the links.
 * - $increment: The increment step (default 10%).
 * - $normal: The textsize normal (default 100%).
 * - $display_current_textsize: Show or hide ("display_hidden") the text of the current textsize.
 * - $current_textsize_text: The current textsize text "Current Size" or "Current Zoom".
 * - $textsize: The current text size (default 100%).
 *
 * @see textsize_theme(), textsize-text.tpl.php
 */
function template_preprocess_textsize_text(&$variables) {
  $textsize_block_title = variable_get('textsize_block_title', 'text_size');
  $textsize_block_title_text = t('Text Size');
  $textsize_current_textsize_text = t('Current Size');
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_increment = variable_get('textsize_increment', 5);
  switch ($textsize_block_title) {
    case 'zoom':
    $textsize_block_title_text = t('Zoom');
    $textsize_current_textsize_text = t('Current Zoom');
    break;
  }
  if (isset($variables['block'])) {
    $variables['template_files'][] = 'textsize-text';
  }
  $variables['block_title'] = $textsize_block_title_text;
  $variables['dest'] = drupal_get_destination();
  $variables['list_inline'] = variable_get('textsize_list_inline', 'list');
  $variables['link_type'] = variable_get('textsize_link_type', 'variable');
  $variables['display_links'] = variable_get('textsize_display_links', 'display');
  $variables['increment'] = textsize_display($textsize_increment, 0);
  $variables['normal'] = textsize_display($textsize_normal, 0);
  $variables['display_current_textsize'] = variable_get('textsize_display_current_textsize', 'display');
  $variables['current_textsize_text'] = $textsize_current_textsize_text;
  $variables['textsize'] = textsize_get_current($value = 'display');
}

/**
 * Process variables for textsize-image.tpl.php.
 *
 * The $variables array contains the following arguments:
 * - $block_title: Block title: "Text Size" or "Zoom".
 * - $dest: The drupal function drupal_get_destination().
 * - $list_inline: Display the ul list "inline" or not.
 * - $link_type: Return "variable" for variable or fix.
 * - $display_links: Show or hide ("display_hidden") the text in the links.
 * - $increment: The increment step (default 10%).
 * - $normal: The textsize normal (default 100%).
 * - $display_current_textsize: Show or hide ("display_hidden") the text of the current textsize.
 * - $current_textsize_text: The current textsize text "Current Size" or "Current Zoom".
 * - $textsize: The current text size (default 100%).
 *
 * @see textsize_theme(), textsize-image.tpl.php
 */
function template_preprocess_textsize_image(&$variables) {
  $textsize_block_title = variable_get('textsize_block_title', 'text_size');
  $textsize_block_title_text = t('Text Size');
  $textsize_current_textsize_text = t('Current Size');
  $textsize_normal = variable_get('textsize_normal', 75);
  $textsize_increment = variable_get('textsize_increment', 5);
  switch ($textsize_block_title) {
    case 'zoom':
    $textsize_block_title_text = t('Zoom');
    $textsize_current_textsize_text = t('Current Zoom');
    break;
  }
  if (isset($variables['block'])) {
    $variables['template_files'][] = 'textsize-image';
  }
  $variables['block_title'] = $textsize_block_title_text;
  $variables['dest'] = drupal_get_destination();
  $variables['list_inline'] = variable_get('textsize_list_inline', 'list');
  $variables['link_type'] = variable_get('textsize_link_type', 'variable');
  $variables['display_links'] = variable_get('textsize_display_links', 'display');
  $variables['increment'] = textsize_display($textsize_increment, 0);
  $variables['normal'] = textsize_display($textsize_normal, 0);
  $variables['display_current_textsize'] = variable_get('textsize_display_current_textsize', 'display');
  $variables['current_textsize_text'] = $textsize_current_textsize_text;
  $variables['textsize'] = textsize_get_current($value = 'display');
}

/**
 * Process variables for textsize-form.tpl.php.
 *
 * The $variables array contains the following arguments:
 * - $textsize_form: The form.
 * - $display_current_textsize: Show or hide ("display_hidden") the text of the current textsize.
 * - $current_textsize_text: The current textsize text "Current Size" or "Current Zoom".
 * - $textsize: The current text size (default 100%).
 *
 * @see textsize_theme(), textsize-form.tpl.php
 */
function template_preprocess_textsize_form(&$variables) {
  $textsize_current_textsize_text = t('Current Size');
  switch ($textsize_block_title) {
    case 'zoom':
    $textsize_current_textsize_text = t('Current Zoom');
    break;
  }
  $variables['textsize'] = array();
  $hidden = array();
  // Provide variables named after form keys so themers can print each element independently.
  foreach (element_children($variables['form']) as $key) {
    $type = $variables['form'][$key]['#type'];
    if ($type == 'hidden' || $type == 'token') {
      $hidden[] = drupal_render($variables['form'][$key]);
    }
    else {
      $variables['textsize'][$key] = drupal_render($variables['form'][$key]);
    }
  }
  // Hidden form elements have no value to themers. No need for separation.
  $variables['textsize']['hidden'] = implode($hidden);
  // Collect all form elements to make it easier to print the whole form.
  $variables['textsize_form'] = implode($variables['textsize']);
  $variables['display_current_textsize'] = variable_get('textsize_display_current_textsize', 'display');
  $variables['current_textsize_text'] = $textsize_current_textsize_text;
  $variables['textsize'] = textsize_get_current($value = 'display');
}
